@@ -1,7 +1,7 @@
 #!/usr/bin/env python
-
+# -*- coding: utf-8 -*-
 # ----------------------------------------------------------------------------
-# Copyright (c) 2011-2013, The BIOM Format Development Team.
+# Copyright (c) 2011-2017, The BIOM Format Development Team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
@@ -9,8 +9,12 @@
 # ----------------------------------------------------------------------------
 
 import os
+import sys
 import inspect
 from contextlib import contextmanager
+import io
+import codecs
+import functools
 
 from collections import defaultdict
 from os import getenv
@@ -18,16 +22,19 @@ from os.path import abspath, dirname, exists
 import re
 from hashlib import md5
 from gzip import open as gzip_open
-from warnings import warn
 
 try:
     import h5py
     HAVE_H5PY = True
-    H5PY_VLEN_STR = h5py.special_dtype(vlen=str)
-    H5PY_VLEN_UNICODE = h5py.special_dtype(vlen=unicode)
+
+    if sys.version_info.major == 2:
+        H5PY_VLEN_STR = h5py.special_dtype(vlen=unicode)  # noqa
+        H5PY_VLEN_UNICODE = h5py.special_dtype(vlen=unicode)  # noqa
+    else:
+        H5PY_VLEN_STR = h5py.special_dtype(vlen=str)
+        H5PY_VLEN_UNICODE = h5py.special_dtype(vlen=str)
 
 except ImportError:
-    warn("h5py is not available")
     HAVE_H5PY = False
     H5PY_VLEN_STR = None
     H5PY_VLEN_UNICODE = None
@@ -35,13 +42,15 @@ except ImportError:
 from numpy import mean, median, min, max
 
 __author__ = "Daniel McDonald"
-__copyright__ = "Copyright 2011-2013, The BIOM Format Development Team"
+__copyright__ = "Copyright 2011-2017, The BIOM Format Development Team"
 __credits__ = ["Daniel McDonald", "Jai Ram Rideout", "Greg Caporaso",
-               "Jose Clemente", "Justin Kuczynski"]
+               "Jose Clemente", "Justin Kuczynski", "Jorge Cañardo Alastuey"]
 __license__ = "BSD"
 __url__ = "http://biom-format.org"
 __maintainer__ = "Daniel McDonald"
 __email__ = "daniel.mcdonald@colorado.edu"
+__format_version__ = (2, 1)
+__version__ = "2.1.6"
 
 
 def generate_subsamples(table, n, axis='sample', by_id=False):
@@ -126,7 +135,7 @@ def unzip(items):
     BSD license).
     """
     if items:
-        return map(list, zip(*items))
+        return list(map(list, zip(*list(items))))
     else:
         return []
 
@@ -321,13 +330,16 @@ def compute_counts_per_sample_stats(table, binary_counts=False):
             sample_counts[sample_id] = (count_vector != 0).sum()
         else:
             sample_counts[sample_id] = float(count_vector.sum())
-    counts = sample_counts.values()
+    counts = list(sample_counts.values())
 
-    return (min(counts),
-            max(counts),
-            median(counts),
-            mean(counts),
-            sample_counts)
+    if len(counts) == 0:
+        return (0, 0, 0, 0, sample_counts)
+    else:
+        return (min(counts),
+                max(counts),
+                median(counts),
+                mean(counts),
+                sample_counts)
 
 
 def safe_md5(open_file, block_size=2 ** 20):
@@ -346,7 +358,7 @@ def safe_md5(open_file, block_size=2 ** 20):
 
     # While a little hackish, this allows this code to
     # safely work either with a file object or a list of lines.
-    if isinstance(open_file, file):
+    if hasattr(open_file, 'read'):
         data_getter = open_file.read
         data_getter_i = block_size
     elif isinstance(open_file, list):
@@ -364,7 +376,7 @@ def safe_md5(open_file, block_size=2 ** 20):
     while data:
         data = data_getter(data_getter_i)
         if data:
-            result.update(data)
+            result.update(data.encode('utf-8'))
     return result.hexdigest()
 
 
@@ -378,7 +390,7 @@ def is_gzip(fp):
     project, but we obtained permission from the authors of this function to
     port it to the BIOM Format project (and keep it under BIOM's BSD license).
     """
-    return open(fp, 'rb').read(2) == '\x1f\x8b'
+    return open(fp, 'rb').read(2) == b'\x1f\x8b'
 
 
 @contextmanager
@@ -410,13 +422,31 @@ def biom_open(fp, permission='U'):
     qiime_open. QIIME is a GPL project, but we obtained permission from the
     authors of this function to port it to the BIOM Format project (and keep it
     under BIOM's BSD license).
+
+    Raises
+    ------
+    RuntimeError
+        If the user tries to parse an HDF5 file without having h5py installed.
+    ValueError
+        If the user tries to read an empty file.
+
     """
     if permission not in ['r', 'w', 'U', 'rb', 'wb']:
         raise IOError("Unknown mode: %s" % permission)
 
-    opener = open
+    opener = functools.partial(io.open, encoding='utf-8')
     mode = permission
 
+    # don't try to open an HDF5 file if H5PY is not installed, this can only
+    # happen if we are reading a file
+    if mode in {'r', 'rb', 'U'}:
+        if os.path.getsize(fp) == 0:
+            raise ValueError("The file '%s' is empty and can't be parsed" % fp)
+
+        if is_hdf5_file(fp) and not HAVE_H5PY:
+            raise RuntimeError("h5py is not installed, cannot parse HDF5 "
+                               "BIOM file")
+
     if HAVE_H5PY:
         if mode in ['U', 'r', 'rb'] and h5py.is_hdf5(fp):
             opener = h5py.File
@@ -425,10 +455,12 @@ def biom_open(fp, permission='U'):
             opener = h5py.File
 
     if mode in ['U', 'r', 'rb'] and is_gzip(fp):
-        opener = gzip_open
+        def opener(fp, mode):
+            return codecs.getreader('utf-8')(gzip_open(fp, mode))
         mode = 'rb' if permission in ['U', 'r'] else permission
     elif mode in ['w', 'wb'] and fp.endswith('.gz'):
-        opener = gzip_open
+        def opener(fp, mode):
+            codecs.getwriter('utf-8')(gzip_open(fp, mode))
 
     f = opener(fp, mode)
     try:
@@ -468,3 +500,21 @@ def get_data_path(fn):
     path = os.path.dirname(os.path.abspath(callers_filename))
     data_path = os.path.join(path, 'test_data', fn)
     return data_path
+
+
+def is_hdf5_file(fp):
+    """Guess if file is HDF5.
+
+    Parameters
+    ----------
+    fn : str
+        File name
+
+    Returns
+    -------
+    bool
+        Whether the file is an HDF5 file
+    """
+    with open(fp, 'rb') as f:
+        # from the HDF5 documentation about format signature
+        return f.read(8) == b'\x89HDF\r\n\x1a\n'
